'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var compositionApi = require('@vue/composition-api');

const key = Symbol('redux');
const install = (app, store) => app.mixin({
  setup() {
    compositionApi.provide(key, store);
    return {};
  }

});

const useStore = () => {
  const store = compositionApi.inject(key);
  assert(store, 'Warning: no redux store was provided.\n\nPlease provide store preferably with vue install\n\napp.use(VueReduxHooks, store)\n\nLearn more about vue-redux-hooks: https://github.com/PatrykWalach/vue-redux-hooks');
  return store;
};

function assert(condition, message) {
  if (!condition) {
    console.error(message);
  }
}

const useSelector = select => {
  const {
    subscribe,
    getState
  } = useStore();
  const selector = compositionApi.ref(select(getState()));
  const unsubscribe = subscribe(() => {
    selector.value = select(getState());
  });
  compositionApi.onUnmounted(unsubscribe);
  return compositionApi.computed(() => selector.value);
};

const useDispatch = () => {
  const {
    dispatch
  } = useStore();
  return dispatch;
};

const VueReduxHooks = {
  install
};

exports.default = VueReduxHooks;
exports.key = key;
exports.useDispatch = useDispatch;
exports.useSelector = useSelector;
exports.useStore = useStore;
//# sourceMappingURL=index.js.map
