import { provide, inject, ref, onUnmounted, computed } from '@vue/composition-api';

const key = Symbol('redux');
const install = (app, store) => app.mixin({
  setup() {
    provide(key, store);
    return {};
  }

});

const useStore = () => {
  const store = inject(key);
  assert(store, 'Warning: no redux store was provided.\n\nPlease provide store preferably with vue install\n\napp.use(VueReduxHooks, store)\n\nLearn more about vue-redux-hooks: https://github.com/PatrykWalach/vue-redux-hooks');
  return store;
};

function assert(condition, message) {
  if (!condition) {
    console.error(message);
  }
}

const useSelector = select => {
  const {
    subscribe,
    getState
  } = useStore();
  const selector = ref(select(getState()));
  const unsubscribe = subscribe(() => {
    selector.value = select(getState());
  });
  onUnmounted(unsubscribe);
  return computed(() => selector.value);
};

const useDispatch = () => {
  const {
    dispatch
  } = useStore();
  return dispatch;
};

const VueReduxHooks = {
  install
};

export default VueReduxHooks;
export { key, useDispatch, useSelector, useStore };
//# sourceMappingURL=index.js.map
